﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Windows;
using System.Windows.Media;
using Infragistics.Windows.DockManager;
using Infragistics.Windows.DockManager.Dragging;

namespace WpfApplication19
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public static string pmLayoutPath = Path.Combine(
            Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
            ApplicationAttributes.CompanyName + "\\" +
            ApplicationAttributes.ProductName + "\\" +
            "MainLayout.lay");

        public MainWindow()
        {
            InitializeComponent();

            DockManager.Loaded += DockManagerOnLoaded;
        }

        private void DockManagerOnLoaded(object sender, RoutedEventArgs routedEventArgs)
        {
            if (File.Exists(pmLayoutPath) != false)
            {
                using (var s = new FileStream(pmLayoutPath, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    //           DockManager.LoadLayout(s);
                }
            }
        }

        private void XamDockManager_PaneDragOver(object sender,
                                                 Infragistics.Windows.DockManager.Events.PaneDragOverEventArgs e)
        {
            if (e.DragAction is MoveWindowAction || e.DragAction is AddToDocumentHostAction ||
                e.DragAction is FloatPaneAction)
                e.IsValidDragAction = true;
            else
                e.IsValidDragAction = false;
        }

        private void XamDockManager_ToolWindowLoaded(object sender,
                                                     Infragistics.Windows.DockManager.Events.PaneToolWindowEventArgs e)
        {
            e.Window.Closing += Window_Closing;

            ContentPane pane = (ContentPane) e.Window.Pane.Panes[0];
            pane.AllowClose = true;
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            PaneToolWindow toolWindow = sender as PaneToolWindow;

            Debug.Assert(toolWindow != null, "toolWindow != null");
            toolWindow.Closing -= Window_Closing;

            // if the pane is a ContentPanePlaceholder then the user is manually dragging the pane back into
            // the tag group.
            if (toolWindow.Pane.Panes[0] is ContentPanePlaceholder)
                return;

            Dispatcher.BeginInvoke(new Action(() =>
                                              {
                                                  // Take the content pane out of the tool window and add it back to the dock manager.
                                                  ContentPane pane = (ContentPane) toolWindow.Pane.Panes[0];

                                                  toolWindow.Pane.Panes.Remove(pane);

                                                  TabPane.Items.Insert(0, pane);
                                                  pane.AllowClose = false;
                                              }));

            e.Cancel = true;
        }

        private void MainWindow_OnClosing(object sender, CancelEventArgs e)
        {
            string dir = Path.GetDirectoryName(pmLayoutPath);

            Debug.Assert(dir != null, "dir != null");
            Directory.CreateDirectory(dir);

            using (var s = new FileStream(pmLayoutPath, FileMode.Create, FileAccess.Write, FileShare.None))
            {
                DockManager.SaveLayout(s);
            }
        }

        public T FindAncestor<T>(DependencyObject dependencyObject) where T : class
        {
            var parent = VisualTreeHelper.GetParent(dependencyObject);

            if (parent == null) return null;

            var parentT = parent as T;

            return parentT ?? FindAncestor<T>(parent);
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            var pti = FindAncestor<PaneTabItem>(sender as DependencyObject);

            ContentPane pane = pti.Pane as ContentPane;
            if (pane != null)
            {
                if (pane.PaneLocation != PaneLocation.Floating)
                {
                    pane.ExecuteCommand(ContentPaneCommands.ChangeToDockable);
                }
            }
        }
    }
}
